diff --git a/choose-mirror.c b/choose-mirror.c
index 6237cfb..389be20 100644
--- a/choose-mirror.c
+++ b/choose-mirror.c
@@ -558,29 +558,33 @@ int get_codename (void) {
 	free(directory);
 	directory = strdup(debconf->value);
 
-	/* As suite has been determined previously, this should not fail */
-	debconf_get(debconf, DEBCONF_BASE "suite");
-	if (strlen(debconf->value) > 0) {
-		suite = strdup(debconf->value);
-
-		asprintf(&command, "wget -q %s://%s%s/dists/%s/Release -O - | grep ^Codename: | cut -d' ' -f 2",
-			 protocol, hostname, directory, suite);
-		di_log(DI_LOG_LEVEL_DEBUG, "command: %s", command);
-		f = popen(command, "r");
-		free(command);
-
-		if (f != NULL) {
-			char buf[SUITE_LENGTH];
-			if (fgets(buf, SUITE_LENGTH - 1, f)) {
-				if (buf[strlen(buf) - 1] == '\n')
-					buf[strlen(buf) - 1] = '\0';
-				debconf_set(debconf, DEBCONF_BASE "codename", buf);
-				di_log(DI_LOG_LEVEL_INFO, "codename set to: %s", buf);
-				ret = 0;
+        /* If preseed specifies codename, omit the codename check */
+	debconf_get(debconf, DEBCONF_BASE "codename");
+	if ( ! strlen(debconf->value) > 0) {
+		/* As suite has been determined previously, this should not fail */
+		debconf_get(debconf, DEBCONF_BASE "suite");
+		if (strlen(debconf->value) > 0) {
+			suite = strdup(debconf->value);
+
+			asprintf(&command, "wget -q %s://%s%s/dists/%s/Release -O - | grep ^Codename: | cut -d' ' -f 2",
+				 protocol, hostname, directory, suite);
+			di_log(DI_LOG_LEVEL_DEBUG, "command: %s", command);
+			f = popen(command, "r");
+			free(command);
+
+			if (f != NULL) {
+				char buf[SUITE_LENGTH];
+				if (fgets(buf, SUITE_LENGTH - 1, f)) {
+					if (buf[strlen(buf) - 1] == '\n')
+						buf[strlen(buf) - 1] = '\0';
+					debconf_set(debconf, DEBCONF_BASE "codename", buf);
+					di_log(DI_LOG_LEVEL_INFO, "codename set to: %s", buf);
+					ret = 0;
+				}
 			}
+			pclose(f);
 		}
-		pclose(f);
-	}
+	} else ret = 0;
 
 	free(hostname);
 	free(directory);
diff --git a/debian/choose-mirror.postinst b/debian/choose-mirror.postinst
index 4a85a78..4689ab0 100644
--- a/debian/choose-mirror.postinst
+++ b/debian/choose-mirror.postinst
@@ -3,10 +3,12 @@
 
 anna-install apt-mirror-setup || true
 
-if [ -f /etc/lsb-release ]; then
-	. /etc/lsb-release
-	if [ -n "$DISTRIB_CODENAME" ]; then
-		db_set mirror/suite "$DISTRIB_CODENAME"
+if ! db_get mirror/suite || ! [ "$RET" ]; then
+	if [ -f /etc/lsb-release ]; then
+		. /etc/lsb-release
+		if [ -n "$DISTRIB_CODENAME" ]; then
+			db_set mirror/suite "$DISTRIB_CODENAME"
+		fi
 	fi
 fi
 
